<?php

	/**
	 * LICENSE: Copyright (c) 2010 François 'cahnory' Germain
	 * Permission is hereby granted, free of charge, to any person obtaining a copy
	 * of this software and associated documentation files (the "Software"), to deal
	 * in the Software without restriction, including without limitation the rights
	 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
	 * copies of the Software, and to permit persons to whom the Software is
	 * furnished to do so, subject to the following conditions:
	 * 
	 * The above copyright notice and this permission notice shall be included in
	 * all copies or substantial portions of the Software.
	 * 
	 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
	 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
	 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
	 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
	 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
	 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
	 * THE SOFTWARE.
	 * that is available through the world-wide-web at the following URI:
	 * http://www.php.net/license/3_01.txt.  If you did not receive a copy of
	 * the PHP License and are unable to obtain it through the web, please
	 * send a note to license@php.net so we can mail you a copy immediately.
	 *
	 * @package    Cahnory
	 * @subpackage Library
	 * @category   Helper
	 * @author     François 'cahnory' Germain <cahnory@gmail.com>
	 * @copyright  2010 François Germain
	 * @license    http://www.opensource.org/licenses/mit-license.php
	 */
	class Helper_Form
	{
		private	$system;
		private	$_sanitize;

		public	function	__construct($system, $config)
		{
			$this->system		=	$system;
		}
		
		public	function	create()
		{
			return	new Helper_Form_Object($system);
		}
	}
	
	class	Form extends Form_FieldCollection
	{
		private	$_success;
		private	$_method	=	'GET';
		private	$_autoFill	=	true;
		private	$_submited;
		
		public	function	__construct()
		{
			$this->setDefinition();
			if($this->_autoFill) {
				if($this->_method == $_SERVER['REQUEST_METHOD']) {
					if($this->_method == 'POST') {
						$this->submit($_POST);
					} else {
						$this->submit($_GET);					
					}
				}
			}
		}
		
		public	function	submit(array $values)
		{
			var_dump($this->toArray());
		}
		
		protected	function	hasMethod($method)
		{
			$this->_method	=	strtoupper($method);
		}
		
		protected	function	autoFill($bool)
		{
			$this->_autoFill	=	$bool;
		}
		
		protected	function	setDefinition() {}	
		protected	function	onSubmit() {}
		protected	function	onSuccess() {}
		protected	function	onError() {}
	}
	
	class	Form_Field
	{
		private	$_directInput	=	true;
		private	$_multiple		=	false;
		private	$_errors		=	array();
		private	$_options		=	array();
		private	$_placeHolder;
		private $_value;
		
		public	function	hasOption($name, $value = NULL)
		{
			$this->_options[]	=	array($name, $value !== NULL ? $value : $name);
		}
		
		public	function	hasOptions(array $options)
		{
			$this->_options		=	array_merge($this->_options, $options);
		}
		
		public	function	isMultiple($bool)
		{
			$this->_multiple	=	(bool)$bool;
		}
		
		public	function	toArray()
		{
			return	array(
				'directInput'	=>	$this->_directInput,
				'multiple'		=>	$this->_multiple,
				'errors'		=>	$this->_errors,
				'options'		=>	$this->_options,
				'placeHolder'	=>	$this->_placeHolder,
				'value'			=>	$this->_value
			);
		}
	}
	
	class	Form_FieldCollection
	{
		private	$_multiple		=	false;
		private	$_collection	=	array();
		
		public	function	__get($name)
		{
			return	isset($this->_collection[$name]) ? $this->_collection[$name] : NULL;
		}
		
		public	function	hasCollection($name, Form_FieldCollection $collection)
		{
			$this->_collection[$name]	=	new Form_FieldCollection;
		}
		
		public	function	hasField($name)
		{
			$this->_collection[$name]	=	new Form_Field;
		}
		
		public	function	isMultiple($bool)
		{
			$this->_multiple	=	(bool)$bool;
		}
		
		public	function	toArray()
		{
			$a	=	array(
				'multiple'		=>	$this->_multiple,
				'fields'		=>	array(),
				'collections'	=>	array()
			);
			foreach($this->_collection as $name => $item) {
				is_a($item, 'Form_FieldCollection')
					?	$a['collections'][$name]	=	$item->toArray()
					:	$a['fields'][$name]			=	$item->toArray();
			}
			return	$a;
		}
	}
	/*
	class Helper_Form_Object
	{
		private	$_tests			=	array();
		private	$_testsArgs		=	array();
		private	$_dependencies	=	array();
		private	$_errorMessages	=	array();
		
		private	$_input			=	array();
		private	$_result		=	array();
		private	$_nbError		=	0;
		
		public	function	__get($name)
		{
			return	isset($this->_result[$name]) ? $this->_result[$name] : array('value' => NULL, 'valid' => NULL, 'message' => NULL);
		}
		
		public	function	nbError()
		{
			return	$this->_nbError;
		}
		
		public	function	process($input)
		{
			$this->_input	=	$input;
			$this->_result	=	array();
			$this->_nbError	=	0;
			foreach($this->_tests as $fieldName => $tests) {
				$this->_result[$fieldName]	=	array(
					'valid'		=>	true,
					'message'	=>	NULL,
					'value'		=>	isset($input[$fieldName]) ? $input[$fieldName] : NULL
				);
				//	Dépendance non satisfaite
				if(isset($this->_dependencies[$fieldName])) {
					foreach($this->_dependencies[$fieldName] as $dependency) {
						if(!isset($this->_result[$dependency]) || !$this->_result[$dependency]['valid']) {
							$this->_result[$fieldName]['valid']	=	NULL;
							continue 2;
						}
					}
				}
				//	Do tests
				foreach($tests as $key => $test) {
					$args	=	$this->_testsArgs[$fieldName][$key];
					array_unshift($args, $this->_result[$fieldName]['value']);
					if(!$this->_result[$fieldName]['valid'] = call_user_func_array(array('Helper_Validate', $test), $args)) {
						//	Test failed
						if(isset($this->_errorMessages[$fieldName][$test])) {
							$this->_result[$fieldName]['message']	=	$this->_errorMessages[$fieldName][$test];
						} elseif(isset($this->_errorMessages[$fieldName]['*'])) {
							$this->_result[$fieldName]['message']	=	$this->_errorMessages[$fieldName]['*'];
						}
						break;
					}
				}
			}
		}
		
		public	function	setRule($field, $tests, $message = NULL, $dependencies = NULL)
		{
			$this->setTest($field, $tests);
			if($message !== NULL)
				$this->setErrorMessage($field, $message);
			if($dependencies !== NULL)
				$this->setDependency($field, $message);
		}
		
		public	function	setTest($field, $tests)
		{
			$tests	=	(array)$tests;
			if(!isset($this->_tests[$field])) {
				$this->_tests[$field]		=	array();
				$this->_testsArgs[$field]	=	array();
			}
			$args	=	func_get_args();
			array_splice($args, 0, 2);
			foreach($tests as $test) {
				if(!in_array($test, $this->_tests[$field])) {
					$this->_tests[$field][]		=	$test;
					$this->_testsArgs[$field][]	=	$args;
				}
			}
		}
		
		public	function	setErrorMessage($field, $message, $tests = '*')
		{
			$tests	=	(array)$tests;
			if(!isset($this->_errorMessages[$field])) {
				$this->_errorMessages[$field]	=	array();
			}
			foreach($tests as $test) {
				$this->_errorMessages[$field][$test]	=	$message;                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       
			}
		}
		
		public	function	setDependency($field, $dependencies)
		{
			$dependencies	=	(array)$dependencies;
			if(!isset($this->_dependencies[$field])) {
				$this->_dependencies[$field]	=	array();
			}
			foreach($dependencies as $dependency) {
				if(!in_array($dependency, $this->_dependencies[$field])) {
					$this->_dependencies[$field][]	=	$dependency;                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       
				}
			}
		}
	}*/

?>